﻿namespace H2Guerilla.Classes
{
    using core.TagInterface;
    using H2Guerilla.Controls;
    using System;
    using System.Windows.Forms;

    internal class TagBlockHelper
    {
        public static Control[] getControls(TagBlock tagBlock, bool Nulled)
        {
            Control[] controlArray = new Control[getNeededControlCount(tagBlock)];
            int index = 0;
            foreach (TagBlockLayout layout in tagBlock.Chunks)
            {
                foreach (BaseObject obj2 in layout)
                {
                    Control control = getControlsFromBaseObject(obj2, Nulled);
                    if (control != null)
                    {
                        controlArray[index] = control;
                        index++;
                    }
                }
            }
            return controlArray;
        }

        public static Control getControlsFromBaseObject(BaseObject baseo, bool Nulled)
        {
            BaseObject baseObject = baseo;
            switch (baseo.Type)
            {
                case BaseObjectType.Angle:
                case BaseObjectType.RawDataOffset:
                case BaseObjectType.LongInteger:
                case BaseObjectType.ShortInteger:
                case BaseObjectType.ByteInteger:
                case BaseObjectType.RealFloat:
                case BaseObjectType.RealFraction:
                case BaseObjectType.String:
                case BaseObjectType.StringID:
                case BaseObjectType.HaloTagType:
                    return new dataType(baseObject, Nulled);

                case BaseObjectType.Flags:
                    return new flags(baseObject, Nulled);

                case BaseObjectType.RealRGBColor:
                case BaseObjectType.RealARGBColor:
                case BaseObjectType.RealRGBAColor:
                case BaseObjectType.RGBAColor:
                    return new colors(baseObject, Nulled);

                case BaseObjectType.Enumerator:
                    return new enums(baseObject, Nulled);

                case BaseObjectType.Padding:
                    return null;

                case BaseObjectType.TagBlock:
                    return new reflexives(baseObject);

                case BaseObjectType.TagIdent:
                    return new idents(baseObject, Nulled);
            }
            return new notSupported(baseObject);
        }

        public static Control[] getControlsFromLayout(TagBlockLayout tagBlock, bool Nulled)
        {
            Control[] controlArray = new Control[getNeededControlCountFromLayout(tagBlock)];
            int index = 0;
            foreach (BaseObject obj2 in tagBlock)
            {
                Control control = getControlsFromBaseObject(obj2, Nulled);
                if (control != null)
                {
                    controlArray[index] = control;
                    index++;
                }
            }
            return controlArray;
        }

        public static int getNeededControlCount(TagBlock tagBlock)
        {
            int num = 0;
            foreach (TagBlockLayout layout in tagBlock.Chunks)
            {
                num += getNeededControlCountFromLayout(layout);
            }
            return num;
        }

        public static int getNeededControlCountFromLayout(TagBlockLayout Layout)
        {
            int num = 0;
            foreach (BaseObject obj2 in Layout)
            {
                if (isAControlNeeded(obj2))
                {
                    num++;
                }
            }
            return num;
        }

        public static bool isAControlNeeded(BaseObject baseObject)
        {
            switch (baseObject.Type)
            {
                case BaseObjectType.RealPoint2D:
                case BaseObjectType.RealPlane2D:
                case BaseObjectType.RealPlane3D:
                case BaseObjectType.RealQuaternion:
                case BaseObjectType.RealPoint3D:
                case BaseObjectType.RealVector3D:
                case BaseObjectType.Angle:
                case BaseObjectType.RealEulerAngles3D:
                case BaseObjectType.Flags:
                case BaseObjectType.RealBounds:
                case BaseObjectType.RealRGBColor:
                case BaseObjectType.RealARGBColor:
                case BaseObjectType.RealRGBAColor:
                case BaseObjectType.RGBAColor:
                case BaseObjectType.Data:
                case BaseObjectType.Animation_Raw:
                case BaseObjectType.Sound_Raw:
                case BaseObjectType.Bitmap_Raw:
                case BaseObjectType.Unicode_Raw:
                case BaseObjectType.RawDataOffset:
                case BaseObjectType.Raw_Tag_Block:
                case BaseObjectType.Enumerator:
                case BaseObjectType.EnumerableIndex:
                case BaseObjectType.TagBlockIndex:
                case BaseObjectType.LongInteger:
                case BaseObjectType.ShortInteger:
                case BaseObjectType.ByteInteger:
                case BaseObjectType.RealFloat:
                case BaseObjectType.RealFraction:
                case BaseObjectType.String:
                case BaseObjectType.StringID:
                case BaseObjectType.Pointer:
                case BaseObjectType.TagBlock:
                case BaseObjectType.TagReference:
                case BaseObjectType.TagIdent:
                case BaseObjectType.HaloTagType:
                case BaseObjectType.Syntax_Block:
                    return true;
            }
            return false;
        }

        public static void reloadControlsWithNewBaseObject(Control container, TagBlock tagBlock, bool Nulled)
        {
            int num = 0;
            for (int i = tagBlock.Chunks.Count - 1; i >= 0; i--)
            {
                for (int j = tagBlock.Chunks[i].Count - 1; j >= 0; j--)
                {
                    BaseObject baseo = tagBlock.Chunks[i][j];
                    if (reloadControlWithBaseObject(container.Controls[num], ref baseo, Nulled))
                    {
                        num++;
                    }
                    if (num == container.Controls.Count)
                    {
                        return;
                    }
                }
            }
        }

        public static void reloadControlsWithNewBaseObject(Control container, TagBlockLayout tagBlock, bool Nulled)
        {
            int num = 0;
            for (int i = tagBlock.Count - 1; i >= 0; i--)
            {
                BaseObject baseo = tagBlock[i];
                if ((container.Controls.Count != 0) && reloadControlWithBaseObject(container.Controls[num], ref baseo, Nulled))
                {
                    num++;
                }
                if (num == container.Controls.Count)
                {
                    return;
                }
            }
        }

        public static bool reloadControlWithBaseObject(Control control, ref BaseObject baseo, bool Nulled)
        {
            switch (baseo.Type)
            {
                case BaseObjectType.Angle:
                case BaseObjectType.RawDataOffset:
                case BaseObjectType.LongInteger:
                case BaseObjectType.ShortInteger:
                case BaseObjectType.ByteInteger:
                case BaseObjectType.RealFloat:
                case BaseObjectType.RealFraction:
                case BaseObjectType.String:
                case BaseObjectType.StringID:
                case BaseObjectType.HaloTagType:
                    ((dataType) control).ReloadWithNewBaseObject(baseo);
                    return true;

                case BaseObjectType.Flags:
                    ((flags) control).ReloadWithNewBaseObject(baseo);
                    return true;

                case BaseObjectType.RealRGBColor:
                case BaseObjectType.RealARGBColor:
                case BaseObjectType.RealRGBAColor:
                case BaseObjectType.RGBAColor:
                    ((colors) control).ReloadWithNewBaseObject(baseo);
                    return true;

                case BaseObjectType.Enumerator:
                    ((enums) control).ReloadWithNewBaseObject(baseo);
                    return true;

                case BaseObjectType.Padding:
                    return false;

                case BaseObjectType.TagBlock:
                    ((reflexives) control).ReloadWithNewBaseObject(baseo, Nulled);
                    return true;

                case BaseObjectType.TagIdent:
                    ((idents) control).ReloadWithNewBaseObject(baseo);
                    return true;
            }
            return true;
        }
    }
}

