﻿namespace H2Guerilla.Classes
{
    using core;
    using core.TagInterface;
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Drawing.Imaging;
    using System.IO;
    using System.Windows.Forms;
    using Xceed.Zip;
    using Xceed.Zip.Sfx;

    public sealed class H2DatabaseHandler
    {
        private static List<string> addedfilepaths;
        private static string databaseDir = "";
        private static string databasePath = "";
        private static DatabaseHeader tempHeader = null;

        public static void CreateDatabase(string InterfacePath, string filePath, DatabaseHeader DatabaseHeader)
        {
            tempHeader = DatabaseHeader;
            DecompiledTagInterface tagInterface = new DecompiledTagInterface(InterfacePath, Map.HaloVersionEnum.Halo2, false);
            CreateDatabaseMeta(InterfacePath, tagInterface, filePath);
        }

        public static void CreateDatabase(string currentInterfacePath, DecompiledTagInterface tagInterface, string filePath, DatabaseHeader DatabaseHeader)
        {
            tempHeader = DatabaseHeader;
            CreateDatabaseMeta(currentInterfacePath, tagInterface, filePath);
        }

        private static void CreateDatabaseMeta(string currentInterfacePath, DecompiledTagInterface tagInterface, string filePath)
        {
            addedfilepaths = new List<string>();
            databasePath = filePath;
            FileInfo info = new FileInfo(filePath);
            databaseDir = info.DirectoryName;
            string str = info.Name.Replace(info.Extension, "");
            StatusHelper.UpdateLabel("Searching for metas...");
            Stack<TagBlock> stack = new Stack<TagBlock>();
            stack.Push(tagInterface.Header);
            while (stack.Count > 0)
            {
                TagBlock block = stack.Pop();
                Application.DoEvents();
                foreach (TagBlockLayout layout in block.Chunks)
                {
                    foreach (BaseObject obj2 in layout)
                    {
                        if (obj2.Type == BaseObjectType.TagReference)
                        {
                            string str5;
                            TagReference reference = (TagReference) obj2;
                            if ((reference.Ident.TagName != "null") && (((str5 = reference.Ident.TagType) == null) || (((str5 != "sncl") && (str5 != "spk!")) && ((str5 != "matg") && (str5 != "ugh!")))))
                            {
                                string item = Tag_Layouts.GetFullTagPath(reference.Ident.TagName, reference.Ident.TagType, Map.HaloVersionEnum.Halo2);
                                Application.DoEvents();
                                if (addedfilepaths.IndexOf(item) == -1)
                                {
                                    if (File.Exists(item))
                                    {
                                        DecompiledTagInterface interface2 = new DecompiledTagInterface(item, Map.HaloVersionEnum.Halo2, false);
                                        addedfilepaths.Add(item);
                                        stack.Push(interface2.Header);
                                    }
                                    else
                                    {
                                        reference.Ident.TagName = "null";
                                    }
                                }
                            }
                            continue;
                        }
                        if (obj2.Type == BaseObjectType.TagIdent)
                        {
                            string str6;
                            TagIdent ident = (TagIdent) obj2;
                            if ((ident.TagName != "null") && (((str6 = ident.TagType) == null) || (((str6 != "sncl") && (str6 != "spk!")) && ((str6 != "matg") && (str6 != "ugh!")))))
                            {
                                string str3 = Tag_Layouts.GetFullTagPath(ident.TagName, ident.TagType, Map.HaloVersionEnum.Halo2);
                                if (addedfilepaths.IndexOf(str3) == -1)
                                {
                                    Application.DoEvents();
                                    if (File.Exists(str3))
                                    {
                                        DecompiledTagInterface interface3 = new DecompiledTagInterface(str3, Map.HaloVersionEnum.Halo2, false);
                                        addedfilepaths.Add(str3);
                                        stack.Push(interface3.Header);
                                    }
                                    else
                                    {
                                        ident.TagName = "null";
                                    }
                                }
                            }
                            continue;
                        }
                        if (obj2.Type == BaseObjectType.Syntax_Block)
                        {
                            string str7;
                            TagIdent ident2 = ((Syntax_Block) obj2).Tag_Reference;
                            if ((ident2.TagName != "null") && (((str7 = ident2.TagType) == null) || (((str7 != "sncl") && (str7 != "spk!")) && ((str7 != "matg") && (str7 != "ugh!")))))
                            {
                                string str4 = Tag_Layouts.GetFullTagPath(ident2.TagName, ident2.TagType, Map.HaloVersionEnum.Halo2);
                                if (addedfilepaths.IndexOf(str4) == -1)
                                {
                                    Application.DoEvents();
                                    if (File.Exists(str4))
                                    {
                                        DecompiledTagInterface interface4 = new DecompiledTagInterface(str4, Map.HaloVersionEnum.Halo2, false);
                                        addedfilepaths.Add(str4);
                                        stack.Push(interface4.Header);
                                    }
                                    else
                                    {
                                        ident2.TagName = "null";
                                    }
                                }
                            }
                            continue;
                        }
                        if (obj2.Type == BaseObjectType.TagBlock)
                        {
                            stack.Push((TagBlock) obj2);
                        }
                    }
                }
            }
            StatusHelper.UpdateLabel("Writing Database...");
            FileStream output = new FileStream(info.DirectoryName + @"\db.tmp", FileMode.Create, FileAccess.Write);
            BinaryWriter bw = new BinaryWriter(output);
            bw.Write((int) (addedfilepaths.Count + 1));
            WriteString(ref bw, currentInterfacePath.Replace(ProgramSettings.DecompiledTagsPath, ""));
            bw.Write((byte) 0);
            for (int i = 0; i < addedfilepaths.Count; i++)
            {
                WriteString(ref bw, addedfilepaths[i].Replace(ProgramSettings.DecompiledTagsPath, ""));
                bw.Write((byte) 0);
            }
            Meta m = TagCompiler.CompileMetaFromTagInterface(tagInterface, 0, 0);
            m.Interface = tagInterface;
            TagDecompiler.Decompile(m, true);
            byte[] buffer = File.ReadAllBytes(currentInterfacePath);
            bw.Write(buffer.Length);
            bw.Write(buffer);
            for (int j = 0; j < addedfilepaths.Count; j++)
            {
                buffer = File.ReadAllBytes(addedfilepaths[j]);
                bw.Write(buffer.Length);
                bw.Write(buffer);
            }
            bw.Close();
            output.Close();
            CreateZip(new string[] { info.DirectoryName + @"\db.tmp" }, str + ".h2gdf");
            File.Delete(databaseDir + @"\db.tmp");
            WriteDatabaseHeader(databasePath);
            StatusHelper.UpdateLabel("Done Creating Database");
            MessageBox.Show("Done");
            StatusHelper.reportProgress.Value = 0;
            StatusHelper.UpdateLabel("Ready");
        }

        private static void CreateZip(string[] FilePaths, string outputPath)
        {
            string zipFileName = outputPath;
            string encryptionPassword = string.Empty;
            EncryptionMethod compatible = EncryptionMethod.Compatible;
            bool replaceExistingFiles = true;
            bool recursive = false;
            bool preservePaths = false;
            long splitSize = 0L;
            SplitNameFormat none = SplitNameFormat.None;
            XceedSfxPrefix sfx = null;
            object userParams = null;
            StatusHelper.reportProgress.Value = 0;
            QuickZip.Zip(zipFileName, encryptionPassword, compatible, replaceExistingFiles, recursive, preservePaths, splitSize, none, sfx, new QuickZip.DiskRequiredCallback(H2DatabaseHandler.OnDiskRequiredCreate), new QuickZip.ByteProgressionCallback(H2DatabaseHandler.OnByteProgressionCreate), new QuickZip.ItemProgressionCallback(H2DatabaseHandler.OnItemProgressionCreate), userParams, FilePaths);
        }

        public static void ExtractDatabaseEnd()
        {
            StatusHelper.reportProgress.Value = 0;
            FileStream input = new FileStream(databaseDir + @"\db.tmp", FileMode.Open, FileAccess.Read);
            BinaryReader br = new BinaryReader(input);
            int max = br.ReadInt32();
            addedfilepaths = new List<string>();
            for (int i = 0; i < max; i++)
            {
                addedfilepaths.Add(ReadString(ref br));
            }
            StatusHelper.SetProgressMax(max);
            bool flag = false;
            for (int j = 0; j < max; j++)
            {
                string path = ProgramSettings.DecompiledTagsPath + addedfilepaths[j];
                StatusHelper.SetProgressValue(j);
                StatusHelper.UpdateLabel("Processing " + path + "...");
                int count = br.ReadInt32();
                byte[] buffer = br.ReadBytes(count);
                if (File.Exists(path) && !flag)
                {
                    if (MessageBox.Show("File already exists would you like to just overwrite all the tags? If you select yes it will overwrite all, otherwise it will message again once it finds another file.", "", MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        flag = true;
                    }
                    else
                    {
                        flag = false;
                        continue;
                    }
                }
                FileStream output = new FileStream(path, FileMode.Create, FileAccess.Write);
                BinaryWriter writer = new BinaryWriter(output);
                writer.Write(buffer);
                writer.Close();
                output.Close();
            }
            br.Close();
            input.Close();
            File.Delete(databaseDir + @"\db.tmp");
            StatusHelper.UpdateLabel("Database Extracted");
            MessageBox.Show("Done");
            StatusHelper.UpdateLabel("Ready");
            StatusHelper.SetProgressValue(0);
        }

        public static void ExtractDatabaseStart(string databasePath)
        {
            int headerSize = ReadDatabaseHeader(databasePath).HeaderSize;
            FileInfo info = new FileInfo(databasePath);
            databaseDir = info.DirectoryName;
            string str = info.Name.Replace(info.Extension, "");
            string path = databaseDir + str + ".tmp1";
            FileStream input = new FileStream(databasePath, FileMode.Open, FileAccess.Read);
            FileStream output = new FileStream(path, FileMode.Create, FileAccess.Write);
            BinaryReader reader = new BinaryReader(input);
            BinaryWriter writer = new BinaryWriter(output);
            reader.BaseStream.Position = headerSize;
            writer.Write(reader.ReadBytes(((int) reader.BaseStream.Length) - headerSize));
            reader.Close();
            writer.Close();
            input.Close();
            output.Close();
            ExtractZip(path, databaseDir);
            ExtractDatabaseEnd();
        }

        public static void ExtractZip(string filePath, string unzipPath)
        {
            string zipFileName = filePath;
            string destinationFolder = unzipPath;
            string decryptionPassword = string.Empty;
            bool replaceExistingFiles = true;
            bool recursive = false;
            bool preservePaths = false;
            object userParams = null;
            string[] filesToUnzip = new string[] { "db.tmp" };
            StatusHelper.reportProgress.Value = 0;
            QuickZip.Unzip(zipFileName, destinationFolder, decryptionPassword, replaceExistingFiles, recursive, preservePaths, new QuickZip.DiskRequiredCallback(H2DatabaseHandler.OnDiskRequired), new QuickZip.ByteProgressionCallback(H2DatabaseHandler.OnByteProgression), new QuickZip.ItemProgressionCallback(H2DatabaseHandler.OnItemProgression), userParams, filesToUnzip);
        }

        public static void OnByteProgression(string currentItemName, long currentItemsByteProcessed, long currentItemTotalBytes, byte currentItemPercent, long allItemsByteProcessed, long allItemsTotalBytes, byte allItemsPercent, object userParams)
        {
            if (((currentItemTotalBytes != 0L) && (allItemsTotalBytes != 0L)) && (allItemsPercent != StatusHelper.reportProgress.Value))
            {
                StatusHelper.reportProgress.Value = allItemsPercent;
            }
        }

        public static void OnByteProgressionCreate(string currentItemName, long currentItemsByteProcessed, long currentItemTotalBytes, byte currentItemPercent, long allItemsByteProcessed, long allItemsTotalBytes, byte allItemsPercent, object userParams)
        {
            if (((currentItemTotalBytes != 0L) && (allItemsTotalBytes != 0L)) && (allItemsPercent != StatusHelper.reportProgress.Value))
            {
                StatusHelper.reportProgress.Value = allItemsPercent;
            }
        }

        public static bool OnDiskRequired(string zipFileName, int diskNumber, object userParams)
        {
            Console.WriteLine(zipFileName + " requires disk #" + diskNumber);
            return true;
        }

        public static bool OnDiskRequiredCreate(string zipFileName, int diskNumber, object userParams)
        {
            Console.WriteLine(zipFileName + " requires disk #" + diskNumber);
            return true;
        }

        public static void OnItemProgression(string currentItemName, long itemProcessed, long totalItemCount, byte totalItemPercent, ref bool abort, object userParams)
        {
            if (itemProcessed != totalItemCount)
            {
                Console.WriteLine(string.Concat(new object[] { "Current item processed: ", currentItemName, " (%", totalItemPercent, ")" }));
            }
        }

        public static void OnItemProgressionCreate(string currentItemName, long itemProcessed, long totalItemCount, byte totalItemPercent, ref bool abort, object userParams)
        {
            if (itemProcessed != totalItemCount)
            {
                StatusHelper.UpdateLabel("Compressing Database...");
            }
        }

        public static DatabaseHeader ReadDatabaseHeader(string databasePath)
        {
            FileStream input = new FileStream(databasePath, FileMode.Open, FileAccess.Read);
            BinaryReader br = new BinaryReader(input);
            DatabaseHeader header = new DatabaseHeader();
            header.ReadHeader(ref br);
            br.Close();
            input.Close();
            return header;
        }

        private static string ReadString(ref BinaryReader br)
        {
            string str = "";
            while (true)
            {
                char ch = br.ReadChar();
                if (ch == '\0')
                {
                    return str;
                }
                str = str + ch;
            }
        }

        private static void WriteDatabaseHeader(string databasePath)
        {
            FileInfo info = new FileInfo(databasePath);
            info.Name.Replace(info.Extension, "");
            FileStream output = new FileStream(databasePath, FileMode.Open, FileAccess.ReadWrite);
            BinaryWriter writer = new BinaryWriter(output);
            BinaryReader reader = new BinaryReader(output);
            MemoryStream stream2 = new MemoryStream();
            BinaryWriter bw = new BinaryWriter(stream2);
            tempHeader.WriteHeader(ref bw);
            bw.Close();
            stream2.Close();
            byte[] buffer = stream2.ToArray();
            byte[] buffer2 = reader.ReadBytes((int) reader.BaseStream.Length);
            writer.BaseStream.Position = 0L;
            writer.Write(buffer);
            writer.Write(buffer2);
            writer.Close();
            reader.Close();
            output.Close();
        }

        private static void WriteString(ref BinaryWriter bw, string WriteString)
        {
            for (int i = 0; i < WriteString.Length; i++)
            {
                bw.Write(WriteString[i]);
            }
        }

        public class DatabaseHeader
        {
            private string author = "";
            private string description = "";
            private int headerSize;
            private System.Drawing.Image image;
            private string version = "";

            public void ReadHeader(ref BinaryReader br)
            {
                this.headerSize = br.ReadInt32();
                this.version = this.ReadString(ref br);
                this.author = this.ReadString(ref br);
                this.description = this.ReadString(ref br);
                int count = br.ReadInt32();
                if (count != -1)
                {
                    MemoryStream stream = new MemoryStream(br.ReadBytes(count));
                    this.image = System.Drawing.Image.FromStream(stream);
                    stream.Close();
                }
            }

            private string ReadString(ref BinaryReader br)
            {
                string str = "";
                while (true)
                {
                    char ch = br.ReadChar();
                    if (ch == '\0')
                    {
                        return str;
                    }
                    str = str + ch;
                }
            }

            public void WriteHeader(ref BinaryWriter bw)
            {
                bw.Write(0);
                this.WriteString(ref bw, this.version);
                this.WriteString(ref bw, this.author);
                this.WriteString(ref bw, this.description);
                if (this.image != null)
                {
                    MemoryStream stream = new MemoryStream();
                    this.image.Save(stream, ImageFormat.Bmp);
                    stream.Close();
                    byte[] buffer = stream.ToArray();
                    bw.Write(buffer.Length);
                    bw.Write(buffer);
                }
                else
                {
                    bw.Write(-1);
                }
                this.headerSize = (int) bw.BaseStream.Position;
                bw.BaseStream.Position = 0L;
                bw.Write(this.headerSize);
            }

            private void WriteString(ref BinaryWriter bw, string WriteString)
            {
                for (int i = 0; i < WriteString.Length; i++)
                {
                    bw.Write(WriteString[i]);
                }
                bw.Write((byte) 0);
            }

            public string Author
            {
                get
                {
                    return this.author;
                }
                set
                {
                    this.author = value;
                }
            }

            public string Description
            {
                get
                {
                    return this.description;
                }
                set
                {
                    this.description = value;
                }
            }

            public int HeaderSize
            {
                get
                {
                    return this.headerSize;
                }
            }

            public System.Drawing.Image Image
            {
                get
                {
                    return this.image;
                }
                set
                {
                    this.image = value;
                }
            }

            public string Version
            {
                get
                {
                    return this.version;
                }
                set
                {
                    this.version = value;
                }
            }
        }
    }
}

