﻿namespace H2Guerilla
{
    using core;
    using core.TagInterface;
    using H2Guerilla.Classes;
    using H2Guerilla.Forms;
    using H2Guerilla.Plugin;
    using System;
    using System.ComponentModel;
    using System.Drawing;
    using System.Windows.Forms;
    using TD.SandDock;

    public class Editor : Form
    {
        private IContainer components;
        private ToolStripMenuItem createDatabaseFromTagToolStripMenuItem;
        private DecompiledTagInterface decompiledTag;
        private DockControl dockControl2;
        private DocumentContainer documentContainer1;
        private string filePath;
        private Map.HaloVersionEnum haloVersion;
        private ToolStrip toolStrip1;
        private ToolStripDropDownButton toolStripDropDownButton1;

        public Editor(TagDefinition TagDefinition)
        {
            base.SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.AllPaintingInWmPaint, true);
            this.InitializeComponent();
            this.CreateMeta(TagDefinition);
            Main.skin.skinChanged += new EventHandler(this.skin_skinChanged);
            this.BackColor = Main.skin.formBackgroundColor;
        }

        public Editor(string FilePath)
        {
            this.filePath = FilePath;
            base.SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.AllPaintingInWmPaint, true);
            this.InitializeComponent();
            this.LoadMeta();
            Main.skin.skinChanged += new EventHandler(this.skin_skinChanged);
            this.BackColor = Main.skin.formBackgroundColor;
        }

        private void createDatabaseFromTagToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new DatabaseCreate(this.filePath, this.decompiledTag).ShowDialog();
        }

        private void CreateMeta(TagDefinition TagDefinition)
        {
            this.decompiledTag = new DecompiledTagInterface();
            this.decompiledTag.Type = Map.HaloVersionEnum.Halo2;
            this.decompiledTag.Tag_Type = TagDefinition.TagType;
            this.decompiledTag.Tag_Definition = TagDefinition;
            this.decompiledTag.Mode = MetaInterface.InterfaceMode.DecompiledTagMode;
            this.decompiledTag.Tag_Name = "";
            this.decompiledTag.Header = new TagBlock(TagDefinition.TagType, TagDefinition.Layout, Map.HaloVersionEnum.Halo2);
            this.decompiledTag.Header.AddNewChunk();
            base.SuspendLayout();
            this.LoadEditors();
            this.dockControl2.Controls.AddRange(TagBlockHelper.getControls(this.decompiledTag.Header, false));
            base.ResumeLayout();
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void InitializeComponent()
        {
            ComponentResourceManager manager = new ComponentResourceManager(typeof(Editor));
            this.documentContainer1 = new DocumentContainer();
            this.dockControl2 = new DockControl();
            this.toolStrip1 = new ToolStrip();
            this.toolStripDropDownButton1 = new ToolStripDropDownButton();
            this.createDatabaseFromTagToolStripMenuItem = new ToolStripMenuItem();
            this.documentContainer1.SuspendLayout();
            this.toolStrip1.SuspendLayout();
            base.SuspendLayout();
            this.documentContainer1.Controls.Add(this.dockControl2);
            this.documentContainer1.Cursor = Cursors.Default;
            this.documentContainer1.Guid = new Guid("7c5a1528-53f0-4697-b4ed-3f358b70dc8c");
            this.documentContainer1.LayoutSystem = new SplitLayoutSystem(250, 400, Orientation.Horizontal, new LayoutSystemBase[] { new DocumentLayoutSystem(0x268, 0x1ce, new DockControl[] { this.dockControl2 }, this.dockControl2) });
            this.documentContainer1.Location = new Point(0, 0x19);
            this.documentContainer1.Manager = null;
            this.documentContainer1.Name = "documentContainer1";
            this.documentContainer1.Size = new Size(0x26a, 0x1d0);
            this.documentContainer1.TabIndex = 4;
            this.dockControl2.AutoScroll = true;
            this.dockControl2.Closable = false;
            this.dockControl2.Collapsible = false;
            this.dockControl2.Guid = new Guid("bd299223-42f3-4477-9541-be24e3f38fd5");
            this.dockControl2.Location = new Point(3, 0x17);
            this.dockControl2.Name = "dockControl2";
            this.dockControl2.Size = new Size(0x264, 0x1b6);
            this.dockControl2.TabIndex = 0;
            this.dockControl2.Text = "Main";
            this.toolStrip1.Items.AddRange(new ToolStripItem[] { this.toolStripDropDownButton1 });
            this.toolStrip1.Location = new Point(0, 0);
            this.toolStrip1.Name = "toolStrip1";
            this.toolStrip1.Size = new Size(0x26a, 0x19);
            this.toolStrip1.TabIndex = 5;
            this.toolStrip1.Text = "toolStrip1";
            this.toolStripDropDownButton1.DisplayStyle = ToolStripItemDisplayStyle.Text;
            this.toolStripDropDownButton1.DropDownItems.AddRange(new ToolStripItem[] { this.createDatabaseFromTagToolStripMenuItem });
            this.toolStripDropDownButton1.Image = (Image) manager.GetObject("toolStripDropDownButton1.Image");
            this.toolStripDropDownButton1.ImageTransparentColor = Color.Magenta;
            this.toolStripDropDownButton1.Name = "toolStripDropDownButton1";
            this.toolStripDropDownButton1.Size = new Size(0x35, 0x16);
            this.toolStripDropDownButton1.Text = "Editors";
            this.createDatabaseFromTagToolStripMenuItem.Name = "createDatabaseFromTagToolStripMenuItem";
            this.createDatabaseFromTagToolStripMenuItem.Size = new Size(0xd7, 0x16);
            this.createDatabaseFromTagToolStripMenuItem.Text = "Create Database From Tag";
            this.createDatabaseFromTagToolStripMenuItem.Click += new EventHandler(this.createDatabaseFromTagToolStripMenuItem_Click);
            base.AutoScaleDimensions = new SizeF(6f, 13f);
            base.AutoScaleMode = AutoScaleMode.Font;
            this.AutoScroll = true;
            base.ClientSize = new Size(0x26a, 0x1e9);
            base.Controls.Add(this.documentContainer1);
            base.Controls.Add(this.toolStrip1);
            base.Name = "Editor";
            this.Text = "Editor";
            this.documentContainer1.ResumeLayout(false);
            this.toolStrip1.ResumeLayout(false);
            this.toolStrip1.PerformLayout();
            base.ResumeLayout(false);
            base.PerformLayout();
        }

        private void LoadEditors()
        {
            foreach (H2GPlugin plugin in H2GPluginHelper.Plugins)
            {
                foreach (string str in plugin.PluginTagType)
                {
                    if ((str == this.decompiledTag.Tag_Type) || (str == "****"))
                    {
                        this.toolStripDropDownButton1.DropDownItems.Add(plugin.PluginName);
                        this.toolStripDropDownButton1.DropDownItems[this.toolStripDropDownButton1.DropDownItems.Count - 1].Click += new EventHandler(this.PluginClick);
                        this.toolStripDropDownButton1.DropDownItems[this.toolStripDropDownButton1.DropDownItems.Count - 1].Tag = plugin;
                    }
                }
            }
        }

        private void LoadMeta()
        {
            this.Text = this.filePath.Replace(Tag_Layouts.GetTagPath(this.haloVersion), "");
            this.decompiledTag = new DecompiledTagInterface(this.filePath, this.haloVersion, false);
            base.SuspendLayout();
            this.LoadEditors();
            foreach (Control control in TagBlockHelper.getControls(this.decompiledTag.Header, false))
            {
                this.dockControl2.Controls.Add(control);
                control.Dock = DockStyle.Top;
                control.BringToFront();
            }
            base.ResumeLayout();
        }

        private void PluginClick(object sender, EventArgs e)
        {
            H2GPlugin tag = (H2GPlugin) ((ToolStripDropDownItem) sender).Tag;
            tag = (H2GPlugin) Activator.CreateInstance(tag.GetType());
            tag.LoadPluginInfo();
            DockControl control = new DockControl();
            control.Text = tag.PluginName;
            tag.Dock = DockStyle.Fill;
            control.Controls.Add(tag);
            this.documentContainer1.AddDocument(control);
            this.documentContainer1.ActiveDocument = control;
            tag.SetTagInterface(this.decompiledTag);
            tag.SetMetaLocation(this.filePath);
            tag.Run();
        }

        public void SaveMeta()
        {
            if ((this.filePath == null) || (this.filePath == ""))
            {
                SaveFileDialog dialog = new SaveFileDialog();
                dialog.Filter = this.decompiledTag.Tag_Definition.TagType + " Meta|*." + this.decompiledTag.Tag_Definition.Extension;
                if (dialog.ShowDialog() != DialogResult.OK)
                {
                    return;
                }
                this.filePath = dialog.FileName;
            }
            Meta m = TagCompiler.CompileMetaFromTagInterface(this.decompiledTag, 0, 0);
            m.Interface = this.decompiledTag;
            TagDecompiler.Decompile(m, true);
            MessageBox.Show("Saved");
        }

        private void skin_skinChanged(object sender, EventArgs e)
        {
            this.BackColor = Main.skin.formBackgroundColor;
        }
    }
}

